#include <stdint.h>
#include <stdio.h>
#include "hid.h"
#include "spi.h"

hid_context device;

/* nRF24L01+ */

#define MY_MAC_0	0xEF
#define MY_MAC_1	0xFF
#define MY_MAC_2	0xC0
#define MY_MAC_3	0xAA
#define MY_MAC_4	0x18
#define MY_MAC_5	0x00

void btLeCrc(const uint8_t* data, uint8_t len, uint8_t* dst)
{
	uint8_t v, t, d;

	while(len--)
	{
		d = *data++;
		for(v = 0; v < 8; v++, d >>= 1)
		{
			t = dst[0] >> 7;
			
			dst[0] <<= 1;
			if(dst[1] & 0x80) dst[0] |= 1;
			dst[1] <<= 1;
			if(dst[2] & 0x80) dst[1] |= 1;
			dst[2] <<= 1;
			
			if(t != (d & 1))
			{
				dst[2] ^= 0x5B;
				dst[1] ^= 0x06;
			}
		}	
	}
}

uint8_t swapbits(uint8_t a)
{
	uint8_t v = 0;
	
	if(a & 0x80) v |= 0x01;
	if(a & 0x40) v |= 0x02;
	if(a & 0x20) v |= 0x04;
	if(a & 0x10) v |= 0x08;
	if(a & 0x08) v |= 0x10;
	if(a & 0x04) v |= 0x20;
	if(a & 0x02) v |= 0x40;
	if(a & 0x01) v |= 0x80;

	return v;
}

void btLeWhiten(uint8_t* data, uint8_t len, uint8_t whitenCoeff)
{
	uint8_t  m;
	
	while(len--)
	{
		for(m = 1; m; m <<= 1)
		{
			if(whitenCoeff & 0x80)
			{
				whitenCoeff ^= 0x11;
				(*data) ^= m;
			}
			whitenCoeff <<= 1;
		}
		data++;
	}
}

static inline uint8_t btLeWhitenStart(uint8_t chan)
{
	//the value we actually use is what BT'd use left shifted one...makes our life easier
	return swapbits(chan) | 2;	
}

void btLePacketEncode(uint8_t* packet, uint8_t len, uint8_t chan)
{
	//length is of packet, including crc. pre-populate crc in packet with initial crc value!
	uint8_t i, dataLen = len - 3;
	btLeCrc(packet, dataLen, packet + dataLen);
	for(i = 0; i < 3; i++, dataLen++) packet[dataLen] = swapbits(packet[dataLen]);
	btLeWhiten(packet, len, btLeWhitenStart(chan));
	for(i = 0; i < len; i++) packet[i] = swapbits(packet[i]);
}

void nrf_cmd(uint8_t cmd, uint8_t data)
{
	spi_select();
	spi_send(cmd);
	spi_send(data);
	spi_unselect();
}

void nrf_simplebyte(uint8_t cmd)
{
	spi_select();
	spi_send(cmd);
	spi_unselect();
}

void nrf_manybytes(uint8_t* data, uint8_t len)
{
	spi_select();
	do {
		spi_send(*data++);
	} while (--len);
	spi_unselect();
}

/* main() */

int main(int argc, char **argv)
{
	int ret;

#if (DATA_SEND == 1)
	int a = 0xAEAEAEAE;
	int b = 0;
#endif
	
	static const uint8_t chRf[] = {2, 26,80};
	static const uint8_t chLe[] = {37,38,39};
	
	uint8_t i, L, ch = 0;
	uint8_t buf[32];
	
	printf("Open STD device...\r\n");
	ret = hid_find(&device, 0x1FC9, 0x0007, 0xFF00, 0);
	
	if (ret == 0)
	{
		//  :
		printf("\r\n");
		printf("ERROR: Device not found!\r\n");
		return 0;
	}
	else
	{
		printf("OK!\r\n");
	}
	
#if defined( ERASE )
	erase_frw();
	return 0;
#endif

	ce_clr();
	
	nrf_cmd(0x20, 0x12);	//on, no crc, int on RX/TX done
	nrf_cmd(0x21, 0x00);	//no auto-acknowledge
	nrf_cmd(0x22, 0x00);	//no RX
	nrf_cmd(0x23, 0x02);	//4-byte address
	nrf_cmd(0x24, 0x00);	//no auto-retransmit
	nrf_cmd(0x26, 0x06);	//1MBps at 0dBm
	nrf_cmd(0x27, 0x3E);	//clear various flags
	nrf_cmd(0x3C, 0x00);	//no dynamic payloads
	nrf_cmd(0x3D, 0x00);	//no features
	nrf_cmd(0x31, 32);		//always RX 32 bytes
	nrf_cmd(0x22, 0x01);	//RX on pipe 0
	
	buf[0] = 0x30;			//set addresses
	buf[1] = swapbits(0x8E);
	buf[2] = swapbits(0x89);
	buf[3] = swapbits(0xBE);
	buf[4] = swapbits(0xD6);
	nrf_manybytes(buf, 5);
	buf[0] = 0x2A;
	nrf_manybytes(buf, 5);

	while(1)
	{
		L = 0;
		
		buf[L++] = 0x42;	//PDU type, given address is random
#if (DATA_SEND == 1)
		buf[L++] = 0x11+10;	//17 bytes of payload
#else
		buf[L++] = 0x11;	//17 bytes of payload
#endif

		buf[L++] = MY_MAC_0;
		buf[L++] = MY_MAC_1;
		buf[L++] = MY_MAC_2;
		buf[L++] = MY_MAC_3;
		buf[L++] = MY_MAC_4;
		buf[L++] = MY_MAC_5;
		
		buf[L++] = 2;		//flags (LE-only, limited discovery mode)
		buf[L++] = 0x01;
		buf[L++] = 0x05;
		
		// name
		buf[L++] = 7;
		buf[L++] = 0x08;
		buf[L++] = 'n';
		buf[L++] = 'R';
		buf[L++] = 'F';
		buf[L++] = ' ';
		buf[L++] = 'L';
		buf[L++] = 'E';
		
#if (DATA_SEND == 1)
		// data
		buf[L++] = 9;
		buf[L++] = 0xff;
		memcpy(&buf[L], &a, 4);
		L += 4;
		memcpy(&buf[L], &b, 4);
		L += 4;
		
		printf("Data: %08x %08x (%d)\r\n", a, b, L);
		b++;
#endif

		buf[L++] = 0x55;	//CRC start value: 0x555555
		buf[L++] = 0x55;
		buf[L++] = 0x55;
		
		printf("BLE full packet size: %d\r\n", L);
		
		if(++ch == sizeof(chRf)) ch = 0;
		
		nrf_cmd(0x25, chRf[ch]);
		nrf_cmd(0x27, 0x6E);	//clear flags
		
		btLePacketEncode(buf, L, chLe[ch]);
		
		nrf_simplebyte(0xE2); // Clear RX Fifo
		nrf_simplebyte(0xE1); // Clear TX Fifo
		
		spi_select();
		spi_send(0xA0);
		for(i = 0 ; i < L ; i++) spi_send(buf[i]);
		spi_unselect();
		
		nrf_cmd(0x20, 0x12);    // tx on
		
		ce_set();               // do tx
		Sleep(10);              // delay_ms(10);
		ce_clr();               // (in preparation of switching to RX quickly)
	}
	
	hid_close(&device);

	return 0;
}
