#include <stdint.h>
#include <stdio.h>
#include "hid.h"
#include "io.h"

extern hid_context device;

/* SPI layer */

void spi_select(void)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_SPI_SELECT;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	//*hw_ver = data_io->dev2pc_hw.hw_ver;
	//return data_io->dev2pc.bRetStatus;
	return;
}

void spi_unselect(void)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_SPI_UNSELECT;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	//*hw_ver = data_io->dev2pc_hw.hw_ver;
	//return data_io->dev2pc.bRetStatus;
	return;
}

void ce_set(void)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_CE_SET;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	//*hw_ver = data_io->dev2pc_hw.hw_ver;
	//return data_io->dev2pc.bRetStatus;
	return;
}

void ce_clr(void)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_CE_CLR;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	//*hw_ver = data_io->dev2pc_hw.hw_ver;
	//return data_io->dev2pc.bRetStatus;
	return;
}

uint8_t spi_send(uint8_t data)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_SPI_SEND;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 1;
	data_io->pc2dev.abData[0] = data;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return 0;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return 0;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	if (data_io->dev2pc.bLen != 1)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return 0;
	}
	//*hw_ver = data_io->dev2pc_hw.hw_ver;
	//return data_io->dev2pc.bRetStatus;
	return data_io->dev2pc.abData[0];
}

int spi_sendn(uint8_t *data, int len)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	if (len > 40)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return -1;
	}
	
	data_io->pc2dev.bOperation = OP_SPI_SEND;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = len;
	memcpy(data_io->pc2dev.abData, data, len);
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return -1;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return -1;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	if (data_io->dev2pc.bLen != len)
	{
		printf("ERROR(%d)!\r\n", __LINE__);
		return -1;
	}
	memcpy(data, data_io->dev2pc.abData, data_io->dev2pc.bLen);
	return 0;
}

/*
	erase_frw()
	   .
 */
int erase_frw(void)
{
	int ret;
	unsigned char buffer[64];
	unsigned char erase_vector[] = FW_CLEAR_VECTOR;
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_FW_CLEAR;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = sizeof(erase_vector);
	memcpy(data_io->pc2dev.abData, erase_vector, sizeof(erase_vector));
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	return data_io->dev2pc.bRetStatus;
}
